classdef CalibNcc
    %CALIBNCC Summary of this class goes here
    %   Detailed explanation goes here
    properties
        fNcc = [];
        fProba = [];
        allNcc = [];
        allOverlap = [];
    end
    
    methods
        function obj = CalibNcc(Img, pos, patchSize)
            pos = round(pos);
    
            % Search area
            xgv = (-patchSize(1)):patchSize(1);
            ygv = (-patchSize(2)):patchSize(2);
            [searchX, searchY] = meshgrid(xgv, ygv);
            searchSize = size(searchX);
            numLoc = numel(searchX);
            nccValues = zeros([searchSize,3], 'single');
            
            % Corresponding overlap
            overlap = (abs(searchX) - patchSize(1)).*(abs(searchY) - patchSize(2)) ...
                ./ (patchSize(1).*patchSize(2));

            % Template pixels
            [templatePixels, tempMaskX, tempMaskY] = app_models.CalibNcc.get_pixels(Img, pos, patchSize);

            for i=1:numLoc
                patchPos = pos + [searchX(i), searchY(i)];
                [patchPixels, patchMaskX, patchMaskY] = app_models.CalibNcc.get_pixels(Img, patchPos, patchSize);
                
                intersX = tempMaskX & patchMaskX;
                intersY = tempMaskY & patchMaskY;
                areaFraction = sum(intersX) * sum(intersY) / (length(intersX) * length(intersY));
                for ch =1:3
                    nccValues((ch-1)*numLoc + i) = areaFraction * ...
                        app_models.CalibNcc.patchSimilarity(...
                        patchPixels(intersX, intersY,ch), ...
                        templatePixels(intersX, intersY,ch));
                end
            end

            % PAV algorithm for monotonic regression
            % --------------------------------------
            nccFinal = nccValues((numLoc+1):(2*numLoc)); %take ch = 2
            [sortedNcc, sortedIdx] = sort(nccFinal, 'ascend');
            sortedNcc = sortedNcc';

            negmask = overlap(:) < 0.7;
            targetW = overlap(:);
            targetW(negmask) = 0.05;

            sortedW = targetW(sortedIdx);
            [oNcc, origIdx] = unique(sortedNcc, 'first');

            % Pool target values corresponding to unique ncc
            poolW = zeros(length(oNcc), 2); % row : avg, totalNum
            lastOrigIdx = [origIdx(2:length(origIdx)) - 1; length(sortedNcc)];
            for j=1:length(oNcc)
                % indices corresponding to this unique value of ncc
                first = origIdx(j);
                last = lastOrigIdx(j);
                % corresponding target value
                targetValues = sortedW(first:last);
                poolW(j,1) = mean(targetValues);
                poolW(j,2) = length(targetValues);
            end

            % Force monotonic constraint
            oProba = zeros(length(oNcc),1);
            oProba(1) = poolW(1,1);

            for j=1:(length(oNcc)-1)
                this_val = oProba(j);
                next_val = poolW(j+1,1);
                if ( this_val <= next_val )
                    oProba(j+1) = next_val;
                    continue
                else
                    avg = sum( poolW(j:(j+1),1).*poolW(j:(j+1),2) )...
                        ./sum( poolW(j:(j+1),2) );
                    oProba(j:(j+1)) = avg;
                    k = j - 1;
                    while (k >= 1)
                        if( oProba(k) <= avg )
                            break
                        else
                            avg = sum( poolW(k:(j+1),1).*poolW(k:(j+1),2) )...
                                ./sum( poolW(k:(j+1),2) );
                            oProba(k:(j+1)) = avg;
                            k = k - 1;
                        end
                    end
                end
            end
            obj.fNcc = oNcc;
            obj.fProba = oProba;
            obj.allNcc = sortedNcc;
            obj.allOverlap = overlap(sortedIdx);
        end% function
        
        function draw(obj, hFigure)
            figure(hFigure);
            set(gca, 'FontSize', 14);
            xlabel('NCC value', 'FontSize', 16);
            ylabel('area overlap', 'FontSize', 16);
            hold on;
            plot(obj.allNcc, obj.allOverlap, 'bo');
            plot(obj.fNcc, obj.fProba, '-r', 'LineWidth', 3);
            xlim([-1 1]); ylim([0 1]);
            hold off;
        end% function
        
        function proba = ncc2proba(obj, testNcc)
            if testNcc < obj.fNcc(1)
                proba = obj.fProba(1);
                return
            end
            ln = length(obj.fNcc);
            if testNcc > obj.fNcc(ln)
                proba = obj.fProba(ln);
                return
            end
            proba = interp1q(obj.fNcc, obj.fProba, testNcc);
        end% function

    end% methods
    
    methods(Static)
        
        function [pixels, maskX, maskY] = get_pixels(Img, pos, patchSize)
            
            pixels = zeros(patchSize(2), patchSize(1), 3);
            
            patchOffset = floor(patchSize./2);
            patchX = (0:(patchSize(1)-1)) - patchOffset(1) + pos(1);
            patchY = (0:(patchSize(2)-1)) - patchOffset(2) + pos(2);
            
            maskX = patchX >= 1.0 & patchX <= size(Img,2);
            maskY = patchY >= 1.0 & patchY <= size(Img,1);
    
            pixels(maskY,maskX,:) = Img(patchY(maskY), patchX(maskX),:);

        end
        
        function ncc = patchSimilarity(patch1, patch2)
            % From Gabe's code for 'plain_ncc'.
            % Evaluates NCC at a single position (scalar product between
            % the two signals)
            
            %numpixel = size(patch1,1) * size(patch1,2);
            T = patch1(:); %((channel-1)*numpixel + 1):(channel*numpixel)); %vector
            F = patch2(:); %((channel-1)*numpixel + 1):(channel*numpixel)); %vector
            
            meanT = sum(T) / length(T);
            Toffset = T - meanT;
            ToffsetSqrd = Toffset .^ 2;
            SumToffsetSqrd = sum(ToffsetSqrd);
            
            meanF = sum(F) / length(F);
            Foffset = F - meanF;
            FoffsetSqrd = Foffset .^ 2;
            SumFoffsetSqrd = sum(FoffsetSqrd);
            
            numerator = sum(Foffset .* Toffset);
            if( numerator == 0 )
                ncc = 0;
            else
                denom = sqrt( SumFoffsetSqrd * SumToffsetSqrd );
                ncc = numerator / denom;
            end
            
        end% function
    end% methods
end% classdef

