function compute_tracking_perfo_on_video(videoID, sDirTracks, sDirGT, ...
    sFnameOutput, bResume)

global logfid
fprintf(logfid, ['\n', videoID, ' :\t']);
disp(videoID)

if ~exist(sDirTracks, 'dir')
    fprintf(logfid, [sDirTracks, ' does not exist.\n']);
    return
end

ofname = [sDirTracks,'/', sFnameOutput, '.mat'];
if exist(ofname, 'file')
    if bResume
        fprintf(logfid, [ofname, ' already exists -> ignoring video.\n']);
        return
    else
        fprintf(logfid, [ofname, ' already exists -> overwriting file. ']);
    end
end

% Look for folders containing tracking results
list = file_manip.get_matching_dirs(sDirTracks, 'mm*'); %see name format line 10
stratsInDir = list(:,2);

% Display number of strategies found in the directory
str = [num2str(numel(stratsInDir)), ' tracking strategies found'];
disp(str);
fprintf(logfid, [str, '\n']);


% ---------------
% Hard-coded info
% ---------------
% File containing the tracking results has this name format:
nameFormat = 'mm %s nl %f';

% We'll consider a track estimate OK if it's within these thresholds of
% precision (in pixels)
okThresholds = 5:15;

% Different performance measures
perfoMeasures = {'%OK', '%Frames', 'AbsOK', 'AbsFrames'};
selectedMeasure = 1; % fraction of OK estimates compared to annotations 

% Perfo plot for this video
visu = false;

% Tracking performances for the different tracking strategies:
trackStrats = struct('model', {}, ...
                     'param', {}, ...
                     'results', {}, ...
                     'avgResults', {});

% ------------------------------
% Process tracks for this video
% ------------------------------            

% In order to check if all GT points are there
GTPoints = file_manip.get_matching_files(sDirGT, 'frame*.txt', 'noR');
trackLength = zeros(size(GTPoints,1), 1);
for iGT = 1:size(GTPoints,1)
     % Get track length
    [dummy, trackLength(iGT)] = annotation.read_annotated_track(...
        [sDirGT, '/',GTPoints{iGT,2}]);
end

for stg = 1:numel(stratsInDir)
    % Which model and params
    scanResult = textscan(stratsInDir{stg}, nameFormat, 'Delimiter', '_');
    trackStrats(stg).model = char(scanResult{1});
    trackStrats(stg).param = scanResult{2};

    % Get tracked points
    fullDir = [sDirTracks,'/', stratsInDir{stg}];
    trackedPoints = file_manip.get_matching_files(fullDir, '*.mat', 'noR');
    trackedPointsFnames = trackedPoints(:,2);
    
    % Ignore those not present in GT
    count = 1;
    while count <= numel(trackedPointsFnames)
        basename = strtok(trackedPointsFnames{count}, '.');
        present = false;
        for iGT = 1:size(GTPoints,1)
            gtBasename = strtok(GTPoints{iGT,2}, '.');
            if strcmp(gtBasename, basename)
                present = true;
                break;
            end
        end
        if present
            count = count + 1;
        else
            trackedPointsFnames(count) = [];
        end
    end
    
    % Check number of tracked points
    if size(GTPoints,1) ~= numel(trackedPointsFnames)
        fprintf(logfid, [videoID, ':\tNot engough points tracked (', ...
            num2str(numel(trackedPointsFnames)), ' out of ', ...
            num2str(size(GTPoints,1)), ')\n']);
    end
    
    
    % Allocate outputs
    trackStrats(stg).results = zeros(...
        numel(trackedPointsFnames),...
        length(perfoMeasures),... %4
        length(okThresholds));
    trackStrats(stg).avgResults = zeros(...
        length(okThresholds), 1);

    % Get perfo for each point
    for iPoint = 1:numel(trackedPointsFnames)
        baseName = strtok( trackedPointsFnames{iPoint}, '.');
        try
            load([fullDir,'/',baseName,'.mat'], 'track');
        catch em
            fprintf(logfid, ['\n', em.message, '\n']);
            continue; 
        end

        [numAnnot, numFrames, okAnnot, okFrames] = ...
            compare_tracks([fullDir,'/',baseName,'.mat'],...
            [sDirGT,'/',baseName,'.txt'], okThresholds);
        trackStrats(stg).results(iPoint,1,:) = okAnnot / numAnnot;
        trackStrats(stg).results(iPoint,2,:) = okFrames / numFrames;
        trackStrats(stg).results(iPoint,3,:) = okAnnot;
        trackStrats(stg).results(iPoint,4,:) = okFrames;
    end

    % Report out result (average over the number of tracked points)
    meanres = squeeze(mean(trackStrats(stg).results,1));
   
    if isnan(sum(meanres(selectedMeasure,:)))
        display (['nan ' stratsInDir{stg} ' ' num2str(videoID)]);
    end
    trackStrats(stg).avgResults = (meanres(selectedMeasure,:))';
            
    % Visualization    
    if visu
        figure; title([trackStrats(stg).model, ' ', ...
            num2str(trackStrats(stg).param)]); ylim([0 1]);
        hold on;
        for perfoMeas = 1:2
            plot(okThresholds, meanres(perfoMeas,:), 'ro:');
        end
        hold off;
    end


end % end num of strategies used in this video

% Save perfos for this video (all motion strategies)
save(ofname, 'trackStrats');
    
end



