function [scores, ok, perfo, sorted_params] = select_perfo(motion_models, trackStrats, varargin)
% SELECT_PERFO(trackStrats, scoreType, selectedMeasure, selectedThreshold, defaultParamValue)
% Params:
%   motion_models: cell array containing the names of hte motin models
%   trackStrats: struct array (row vector) containing performances for different motion
%      models and parameters.
%   scoreType: 'default', 'best', '3best', 'area'
%   selectedMeasure : index of perfoMeasures = {'%OK', '%Frames', 'AbsOK', 'AbsFrames'}. 
%      Example: selectedMeasure = 1 corresponds to fraction of OK estimates
%      compared to annotations
%   selectedThreshold: inde of okThresholds = 5:15. Example: selectedTh = 6 means th=10.
%   defaultParamValue: param vamue used idf scoreType = 'default'


% Input args
scoreType = 'default';
selectedMeasure = 1;
selectedTh = 6;
defaultParam = 1;

argi = 1;

if numel(varargin) >= argi
    scoreType = varargin{argi};
end
argi = argi + 1;
if numel(varargin) >= argi
    selectedMeasure = varargin{argi};
end
argi = argi + 1;
if numel(varargin) >= argi
    selectedTh = varargin{argi};
end
argi = argi + 1;
if numel(varargin) >= argi
    defaultParam = varargin{argi};
end

clear argi;
    
% Output
scores = zeros(1, numel(motion_models)); % one score per motion model
ok = true;
perfo = {};
sorted_params = {};

% For each motion model
for mm = 1:numel(motion_models)
 
    % Group all the motion strategies corresponding to THE SAME motion
    % model (different parameters)
    numTrackStrats = numel(trackStrats);
    tmpvar = cell(1,numTrackStrats);
    ct_val = motion_models(mm);
    tmpvar(1:numTrackStrats) = ct_val;
    match = cellfun(@strcmp,{trackStrats(1,:).model},tmpvar);
   
    if numel(match) <= 0
        display([motion_models{mm}, ': No matching elements found']);
        scores(mm) = -1;
        ok = false;
        continue;
    end
    
    matchStrat = trackStrats(1,match);
    %(matchStrat contains only the elements that match the given
    % motion model, e.g. brownian)
    
    % Sort the parameter values
    [sortedParams, sortidx] = sort(cell2mat({matchStrat(1,:).param}));
    numParams = length(sortidx);
    
    display([motion_models{mm}, ': ', num2str(numParams), ' parameters found']);

    % Get average results (over all tracked points) for this video
    perfo{mm} = zeros(1, numParams);
    sorted_params{mm} = sortedParams;
    
    for pp = 1:numParams
        perfo{mm}(pp) = mean(matchStrat(sortidx(pp)).results(:,selectedMeasure, selectedTh));
        % check
        if abs(perfo{mm}(pp) - matchStrat(sortidx(pp)).avgResults(selectedTh)) > 0.00001
            display('WARNING: non matching scores for default selected measure')
            ok = false;
        end
    end
    sortedPerfos = sort(perfo{mm}, 'descend');
    
    % Compute score for this motion model on this video depending on scoreType
    if strcmp(scoreType, 'default')
        % Perfo obtained with default param
        scores(1,mm) = perfo{mm}(sortedParams == defaultParam);
    elseif strcmp(scoreType, 'best')
        % Perfo obtained with best param for this motion model
        scores(1,mm) = sortedPerfos(1);
    elseif strcmp(scoreType, '3best')
        % Average perfo of three best params
        scores(1,mm) = mean(sortedPerfos(1:3));
    elseif strcmp(scoreType, 'area')
        % Area under the curve
        scores(1,mm) = area_under_curve(sortedParams, perfo(mm,:))...
            / (sortedParams(numParamValues) - sortedParams(1));
    else
        display(['unknown scoreType ', scoreType]);
        ok = false;
    end
    
end


end


