function track_video(iStartFrame, iEndFrame, sFrameNamePattern, ...
    sMotionType, fNoiseLevel, bFlipVideo, sRunTag, bRiskyResume)
%TRACK_VIDEO Get input frames and interest points and track them. 
% 
% Input and output directories, patch size, number of particles,
% stopping conditions, etc. are hard-coded.

global GDataDir

% VideoID of the original video
videoID = strtok(sFrameNamePattern, '%'); 
% VideoID of the flipped video
videoID_flip = [videoID, '_FLIP_'];

% Input points and PNGs
% ---------------------
% The PNGs are those of the original video
pngDir = [GDataDir, '/frames/', videoID];
% The GT points are at the original video's Data directory
pointsDir = [GDataDir, '/GT_tracks/', videoID, '/', videoID 'Data'];

% Output tracks
% -------------
% Hard-coded directories
if bFlipVideo
    videoDir_flip = [GDataDir, '/tracks/', videoID_flip];
    mkdir(videoDir_flip);
    fileattrib(videoDir_flip, '+w', 'g', 's');
    tracksDir = [videoDir_flip, '/', videoID_flip, 'Tracks', sRunTag];
else
    tracksDir = [GDataDir, '/tracks/', videoID, '/', videoID, 'Tracks', sRunTag];
end

if ~exist(tracksDir, 'dir')
    mkdir(tracksDir);
    fileattrib(tracksDir, '+w', 'g', 's');
end

theseTracksDir = [tracksDir, '/mm_', sMotionType, '_nl_', num2str(fNoiseLevel)];
if ~exist(theseTracksDir, 'dir')
    mkdir(theseTracksDir);
    fileattrib(theseTracksDir, '+w', 'g', 's');
else
    if bRiskyResume 
        % Quit without checking the contents of the folder
        return
    end
end

% Patch size
% ----------
szPatch = [15, 15];
szPatchOffset = floor(szPatch./2);
% Get mesh grid centered at the template
xgv = (0:(szPatch(1)-1)) - szPatchOffset(1);
ygv = (0:(szPatch(2)-1)) - szPatchOffset(2);
[templX, templY] = meshgrid(xgv, ygv);

imgWidth = [];
imgHeight = [];

% Stoping condition
% -----------------
stopNframes = inf;
stopThreshold = 0.3;

% Number of particles per tracked object
% --------------------------------------
numParticles = 50;

% Motion model
% ------------
MotionModel = motion_models.init_motion_model(sMotionType, fNoiseLevel);
numDims = MotionModel.numDims;

% Tracked objects
% ---------------
objProto = struct(...
    'id', 0, ...
    'initPos', zeros(1,3,'uint16'), ...
    'template', zeros([szPatch(2), szPatch(1), 3], 'single'), ...
    'particles', zeros(numParticles, MotionModel.numDims, 'single'),...
    'weights', (1/numParticles)*ones(numParticles,1, 'single'),...
    'idxToPropagate', 1:uint16(numParticles), ...
    'appModel', [], ...
    'resample', false, ...
    'prediction', zeros(iEndFrame-iStartFrame, 3), ...
    'timeOfLife', uint16(0), ...
    'end', uint8(0), ...
    'debugMaxLife', uint16(0));
Objects = struct(objProto);
uniqueId = uint16(0);
countObj = uint16(0);

% Figures for visualization
% -------------------------
szScr = get(0, 'ScreenSize');
hVisu = figure('Position', [5, 5, szScr(3)*0.9, szScr(4)*0.9]);
%jFrame = get(handle(gcf),'JavaFrame');
%jFrame.setMinimized(true);   % to minimize the figure

for iFrame = iStartFrame:iEndFrame

    % Load current frame 
    % -------------------
    frameName = sprintf(sFrameNamePattern, iFrame);
    Img = single(imread([pngDir, '/', frameName, '.png']));
    if isempty(imgWidth)
        imgWidth = size(Img, 2);
        imgHeight = size(Img, 1);
    end
    % FLIP?
    if bFlipVideo
        Img(:,:,:) = Img(:,(imgWidth:-1:1),:);
    end
    % Show
    set(0,'CurrentFigure',hVisu);
    image(Img/255); axis image;
   
    
    %======================================================================
    % Particle filter iteration for each tracked object
    % -------------------------------------------------
    for iObj = 1:countObj
        
        % Resample particles according to their likelihood
        % ------------------------------------------------
        if Objects(iObj).resample
            Objects(iObj).idxToPropagate = resample_particles(...
                Objects(iObj).weights, numParticles);
        else
            Objects(iObj).resample = true;
        end
    
        % Propagate existing particles to current frame
        % ---------------------------------------------
        newParticles = zeros(numParticles, size(Objects(iObj).particles, 2));
        for iSampl=1:length(Objects(iObj).idxToPropagate)
            idx = Objects(iObj).idxToPropagate(iSampl);
            newParticles(iSampl,:) = MotionModel.predict(...
                Objects(iObj).particles(idx,:));
        end
        Objects(iObj).particles = newParticles;
        clear newParticles;
        
        % Measure
        % -------
        if numParticles ~= length(Objects(iObj).weights)
            Objects(iObj).weights = zeros(numParticles,1);
        end
        for iPart = 1:numParticles
            % Scaling and translation wrt initial template
            sWH = Objects(iObj).particles(iPart,3:4) ./ szPatch;
            scaleMat = diag(sWH);
            translationXY = Objects(iObj).particles(iPart,1:2);
            % Get patch from current image after transformation of template
            % coordinates
            Tform = maketform('affine', [scaleMat; translationXY]);             
            [imgX, imgY] = tformfwd( Tform, templX(:), templY(:) );
            clear sWH scaleMat translationXY Tform;
            
            % Check that the predicted location is a place we can really
            % evaluate the likelihood.
            inFrame = min(imgY) >= 1.0   &&  max(imgY) <= imgHeight && ...
                min(imgX) >= 1.0   &&  max(imgX) <= imgWidth;
            patch = zeros([szPatch,3]);
            if inFrame
                imgX = reshape(imgX, szPatch);
                imgY = reshape(imgY, szPatch);
                %patch(:,:,1) = interp2(Img(:,:,1), imgX, imgY, '*linear');
                patch(:,:,2) = interp2(Img(:,:,2), imgX, imgY, '*linear');
                %patch(:,:,3) = interpg(Im2(:,:,3), imgX, imgY, '*linear');
                ch = 2;
                ncc = app_models.CalibNcc.patchSimilarity(patch(:,:,ch),...
                    Objects(iObj).template(:,:,ch));
                Objects(iObj).weights(iPart) = Objects(iObj).appModel.ncc2proba(ncc);
            else
                Objects(iObj).weights(iPart) = 0.0;
            end
            Objects(iObj).particles(iPart,numDims) = Objects(iObj).weights(iPart);
            
            clear imgX imgY patch;
        end
        
        
        % Report our predicted position for this frame
        % --------------------------------------------
        Objects(iObj).timeOfLife = Objects(iObj).timeOfLife + 1;
        [maxLike, maxIdx] = max( Objects(iObj).weights );
        maxParticle = Objects(iObj).particles(maxIdx, :);
        Objects(iObj).prediction(Objects(iObj).timeOfLife, :) = [maxParticle(1:2), maxLike];
        % Stop condition
        if maxLike < stopThreshold
            Objects(iObj).end = Objects(iObj).end + 1;
        end
        clear maxParticle
        
        % Normalize weights
        % -----------------
        sumWeights = sum(Objects(iObj).weights);
        if  sumWeights == 0
            Objects(iObj).weights = ones(numParticles,1) ./ numParticles;
        else
            Objects(iObj).weights = Objects(iObj).weights ./ sumWeights;
        end
        % Effective number of particles
        %Neff = 1 ./ sum( Objects(iObj).weights .* Objects(iObj).weights );    
        
    end
    %======================================================================
    
    % Stop tracking some objects
    % --------------------------
    iObj = uint16(1);
    while iObj <= countObj
        if Objects(iObj).end > stopNframes ...
                || Objects(iObj).timeOfLife >= Objects(iObj).debugMaxLife
            % Save sth somewhere
            annotation.save_track(theseTracksDir, Objects(iObj));
            % Delete
            Objects(iObj) = [];
            countObj = countObj - 1;
        else
           iObj = iObj + 1; 
        end
    end
    
    % Display current tracks
    % ----------------------
    set(0,'CurrentFigure',hVisu);
    hold on
    for iObj = 1:countObj
        
        % plot particles
        plot(Objects(iObj).particles(:,1), Objects(iObj).particles(:,2), 'cx')
        
        % plot predicted particle
        last = Objects(iObj).timeOfLife;
        plot(Objects(iObj).prediction(last,1), Objects(iObj).prediction(last,2), ...
            'bo','LineWidth',2,'MarkerSize',5);
        
        % plot line linking last position with current position
        if last == 1
            xy = [Objects(iObj).initPos(1:2); Objects(iObj).prediction(1,1:2)];
            plot(xy(:,1), xy(:,2), 'g-');
            clear xy
        else
            plot(Objects(iObj).prediction((last-1):last,1), Objects(iObj).prediction((last-1):last,2), 'g-');
        end
        
    end
    hold off
    drawnow

    
    % Add points to the list of objects to track in the next frame
    % ------------------------------------------------------------
    
    [Points, maxLives] = annotation.search_points_to_track(pointsDir, ...
        theseTracksDir, iFrame, bFlipVideo, imgWidth);
    if ~isempty(Points)
        numNewObjects = uint16(size(Points,1));
        % pre-allocation
        Objects((countObj + 1):(countObj + numNewObjects)) = struct(objProto);
        % init of each object
        for iPoint = 1:numNewObjects
            iObj = countObj + iPoint;
            Objects(iObj).id = uniqueId + iPoint;
            pos = Points(iPoint,:);
            Objects(iObj).initPos = uint16([pos, iFrame]);
            
            % Correction if the patch is partly outside the image
            Objects(iObj).template= ...
                app_models.CalibNcc.get_pixels(Img, pos, szPatch);
            
            Objects(iObj).particles = MotionModel.init_particles(...
                [pos, szPatch], numParticles, imgWidth, imgHeight);
            
            Objects(iObj).appModel = app_models.CalibNcc(Img, pos, szPatch);
            
            Objects(iObj).debugMaxLife = maxLives(iPoint);
            
        end
        countObj = countObj + numNewObjects;
        uniqueId = uniqueId + numNewObjects;
        
        % Show
        set(0,'CurrentFigure',hVisu);
        hold on;
        plot(Points(:,1), Points(:,2), 'ro', 'LineWidth',2,'MarkerSize',10);
        hold off;
        drawnow
        
    end
      
      
end% for frames


for iObj = 1:countObj
    % Save sth somewhere
    annotation.save_track(theseTracksDir, Objects(iObj));
end


fileattrib([tracksDir '/*'], '+w', 'g', 's');

end% function

